/*
 *  qrb.c - computing distance between two locators
 *  Copyright (c)2002-2004 Andrej OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009-2011 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "osdep.h"
#include "debug.h"
#include "qrb.h"
#include "main.h"
#include "load.h"


//#define PI 3.1415927

#if 0
double acs(/*uint32_t*/ double arg)
{
	DEBUG_FUNC("acs");

	if (arg == 0)
		return (MY_PI / 2);
	else if (arg < 0)
		return (MY_PI + atan(sqrt(1 - arg * arg) / arg));
	else
		return (atan(sqrt(1 - arg * arg) / arg));
}
#endif

double dely(uint32_t num) /* y */
{
	DEBUG_FUNC("delx");

	return (((double) (10 * ((num >> 18) & 0x1F) + ((num >> 10) & 0x0F) +
					   (((double) (num & 0x1F))+0.5) / 24)) * MY_PI / 180);
	/* (((double) (10 * (loc[1] - 'A') + (loc[3] - '0') + ((double) (loc[5] - 'A')) / 24))
	   * MY_PI / 180); */
}

double delx(uint32_t num) /* x */
{
	DEBUG_FUNC("dely");

	return (((double)
			 (20 * ((num >> 23) & 0x1F) + 2 * ((num >> 14) & 0x0F) +
			  (((double) ((num >> 5) & 0x1F))+0.5) / 12)) * MY_PI / 180);
	/* (((double) (20 * (loc[0] - 'A') + 2 * (loc[2] - '0') + ((double) (loc[4] - 'A')) / 12))
	   * MY_PI / 180); */
}

static int compute_qrb(uint32_t loc1, uint32_t loc2)
{

	if (loc1 == loc2)
		return 1;

	double x1= delx(loc1);
	double y1= dely(loc1);
	double x2 = delx(loc2);
	double y2 = dely(loc2);

	if (!(loc1 & 0x80000000))
		return 0;
	if (!(loc2 & 0x80000000))
		return 0;

	double dis = /*6365*/ R_EARTH * acos(cos(y1) * cos(y2) + sin(y1) * sin(y2) * cos(x2 - x1));

/*	double zv = dis - floor(dis);

	if (zv > 0.5 && zv != 0)
		dis++;*/
		

/* printf("x1=%7.8f, y1=%7.8f, x2=%7.8f, y2=%7.8f, points=%u, qrb=%7.2f\n", x1, y1, x2, y2, points, dis); */
	return (int) dis;
//	return points;
}

static int qthwr(char *qth,int width) {
    char *c,d;
    int res;
    
    c=qth;
    while (1){
        d=tolower(*c);
        if (!d) return(-1000);
        if (d>='a' || d<='r') break;
        c++;
    }
    if (width) c++;
    d=tolower(*c);
    if (strlen(c)<5) return(-1000);
    
    res=(d-106)*10;
    c+=2;

    if (!isdigit(*c)) return(-1000);
    res+=(*c-'0');
    
    return(res);
}

static int qsopwr(char *myqth, char *recqth){
    int h1, h2, w1, w2, dh1, dh2, dh, dw, d;
    
    if ((h1=qthwr(myqth,0))<=-1000) return(-1);
    if ((w1=qthwr(myqth,1))<=-1000) return(-1);
    if ((h2=qthwr(recqth,0))<=-1000) return(-1);
    if ((w2=qthwr(recqth,1))<=-1000) return(-1);

    /*dbg("h1=%d  w1=%d  h2=%d  %w2=%d \n",h1,w1,h2,w2);*/

    
    dh1 = h1 - h2; if (dh1 < 0) dh1 += 180; 
    dh2 = h2 - h1; if (dh2 < 0) dh2 += 180; 
/*    dw1 = w1 - w2; if (dw1 < 0) dw1 += 180; 
    dw2 = w2 - w1; if (dw2 < 0) dw2 += 180; */

    dh = dh1 < dh2 ? dh1 : dh2;
/*    dw = dw1 < dw2 ? dw1 : dw2;*/
    dw = w2 - w1;
    if (dw < 0) dw = -dw;

    d = dh < dw ? dw : dh;
    
    return d+2;
}

void recalc_stats(struct QSO_INFO *q, uint32_t myloc){
    char myloc_s[10];
    char loc_s[10];

    if (cwonly){
        //intf("CWONLY %s %s %s\n", q->r_rst, q->s_rst, q->ok_rst);
        if (strlen(q->r_rst) != 3 ||
            strlen(q->s_rst) != 3 ||
            strlen(q->ok_rst) != 3 ) {
            //q->qrb = 0;
            //q->points = 0;
            //q->claimed_points = 0;
            q->dupe = 2;
            return;
        }    
    }

    q->qrb = compute_qrb(myloc, q->r_call->loc);
    

    switch (qsop){
        case 4:
            num2loc(myloc, myloc_s);
            num2loc(q->r_call->loc, loc_s);
            q->points = qsopwr(myloc_s, loc_s); 
            break;
        case 1:
        default:
    	    q->points = (int)(q->qrb + 1.0); /* distance to points rounding - as specified by IARU */
            break;
    }

}
